@@ -135,7 +135,7 @@ static inline clash_action ask_namematch(doscp_t *cp,
 	/**
 	 * Name to be printed
 	 */
-	char *name;
+	const char *name;
 
 #define EXISTS 0
 #define RESERVED 1
@@ -316,7 +316,7 @@ static int contains_illegals(const char *string, const char *illegals,
 	return 0;
 }
 
-static int is_reserved(char *ans, int islong)
+static int is_reserved(const char *ans, int islong)
 {
 	unsigned int i;
 	static const char *dev3[] = {"CON", "AUX", "PRN", "NUL", "   "};
@@ -383,7 +383,8 @@ static inline clash_action get_slots(Stream_t *Dir,
 					ch->source_entry,
 					pessimisticShortRename &&
 					ch->use_longname,
-					ch->use_longname)) {
+					ch->use_longname,
+					ch->is_label)) {
 			case -1:
 				return NAMEMATCH_ERROR;
 
@@ -530,14 +531,14 @@ static void stripspaces(char *name)
 }
 
 
-static int _mwrite_one(Stream_t *Dir,
-		       char *argname,
-		       char *shortname,
-		       write_data_callback *cb,
-		       void *arg,
-		       ClashHandling_t *ch)
+static int mt_mwrite_one(Stream_t *Dir,
+			 char *argname,
+			 char *shortname,
+			 write_data_callback *cb,
+			 void *arg,
+			 ClashHandling_t *ch)
 {
-	char longname[VBUFSIZE];
+	char longname[MAX_VNAMELEN_MB+1];
 	const char *dstname;
 	dos_name_t dosname;
 	int expanded;
@@ -576,7 +577,7 @@ static int _mwrite_one(Stream_t *Dir,
 	}
 
 	/* Copy original argument dstname to working value longname */
-	strncpy(longname, dstname, VBUFSIZE-1);
+	strncpy(longname, dstname, MAX_VNAMELEN_MB);
 
 	if(shortname) {
 		ch->use_longname =
@@ -662,7 +663,7 @@ int mwrite_one(Stream_t *Dir,
 		shortname = strdup(_shortname);
 	else
 		shortname = 0;
-	ret = _mwrite_one(Dir, argname, shortname, cb, arg, ch);
+	ret = mt_mwrite_one(Dir, argname, shortname, cb, arg, ch);
 	if(argname)
 		free(argname);
 	if(shortname)
