@@ -55,7 +55,7 @@ struct vfat_subentry {
 struct vfat_state {
 	wchar_t name[VBUFSIZE];
 	int status; /* is now a bit map of 32 bits */
-	int subentries;
+	unsigned int subentries;
 	unsigned char sum; /* no need to remember the sum for each entry,
 			    * it is the same anyways */
 	int present;
@@ -64,6 +64,15 @@ struct vfat_state {
 const char *short_illegals=";+=[]',\"*\\<>/?:|";
 const char *long_illegals = "\"*\\<>/?:|\005";
 
+/* Format a number into a string, without terminating the string */
+static void fmt_num(unsigned int num, char *base, int end, char prefix) {
+	for(;end;end--) {
+		base[end-1] = '0' + num % 10;
+		num = num / 10;
+	}
+	base[end] = prefix;
+}
+
 /* Automatically derive a new name */
 static void autorename(char *name,
 		       char tilda, char dot, const char *illegals,
@@ -71,7 +80,6 @@ static void autorename(char *name,
 {
 	int tildapos, dotpos;
 	unsigned int seqnum=0, maxseq=0;
-	char tmp;
 	char *p;
 
 #ifdef DEBUG
@@ -85,6 +93,8 @@ static void autorename(char *name,
 			bump = 0;
 		}
 
+	/* First step: find out whether the proposed name already ends
+	   in a sequence number, and fill in tildapos if so */
 	for(dotpos=0;
 	    name[dotpos] && dotpos < limit && name[dotpos] != dot ;
 	    dotpos++) {
@@ -96,10 +106,12 @@ static void autorename(char *name,
 			seqnum = seqnum * 10 + (uint8_t)(name[dotpos] - '0');
 			maxseq = maxseq * 10;
 		} else
-			tildapos = -1; /* sequence number interrupted */
+			tildapos = -1; /* "sequence number" followed
+					* by non-numeric characters,
+					* i.e. consider as not present */
 	}
 	if(tildapos == -1) {
-		/* no sequence number yet */
+		/* if no sequence number found at step 1, make space for one */
 		if(dotpos > limit - 2) {
 			tildapos = limit - 2;
 			dotpos = limit;
@@ -114,23 +126,25 @@ static void autorename(char *name,
 		if(seqnum > 999999) {
 			seqnum = 1;
 			tildapos = dotpos - 2;
-			/* this matches Win95's behavior, and also guarantees
-			 * us that the sequence numbers never get shorter */
+			/* produces a short sequence number within the former
+			 * big sequence number: A~9999~1 TST */
 		}
 		if (seqnum == maxseq) {
-		    if(dotpos >= limit)
-			tildapos--;
-		    else
-			dotpos++;
+			/* series of nines => make space for one more
+			 * digit */
+			if(dotpos >= limit)
+				tildapos--;
+			else
+				dotpos++;
 		}
 	}
 
-	tmp = name[dotpos];
 	if((bump && seqnum == 1) || seqnum > 1 || mtools_numeric_tail)
-		sprintf(name+tildapos,"%c%d",tilda, seqnum);
-	if(dot)
-	    name[dotpos]=tmp;
-	/* replace the character if it wasn't a space */
+		fmt_num(seqnum, name+tildapos, dotpos-tildapos, tilda);
+
+	/* terminate the string if long name */
+	if(!dot)
+		name[dotpos]='\0';
 #ifdef DEBUG
 	printf("Out autorename for name=%s.\n", name);
 #endif
@@ -271,54 +285,9 @@ static inline void check_vfat(struct vfat_state *v, struct directory *dir)
 	v->present = 1;
 }
 
-#ifdef HAVE_PRAGMA_DIAGNOSTIC
-# pragma GCC diagnostic push
-# pragma GCC diagnostic ignored "-Wsign-conversion"
-#endif
-
-/* We have indeed different types for the entry slot
- * - the higher levels have a "signed" type, in order to accomodate
- *   reserved values for "root directory" entry, "not found" entries, and
- *   "uninitialized"
- * - the lower levels always consider it as an index into the
- *   directory viewed as a table, i.e. always positive
- */
-int clear_vses(Stream_t *Dir, int entrySlot, unsigned int last)
-{
-	direntry_t entry;
-	dirCache_t *cache;
-	int error;
-
-	entry.Dir = Dir;
-	entry.entry = entrySlot;
-
-	/*maximize(last, entry.entry + MAX_VFAT_SUBENTRIES);*/
-	cache = allocDirCache(Dir, last);
-	if(!cache) {
-		fprintf(stderr, "Out of memory error in clear_vses\n");
-		exit(1);
-	}
-	addFreeEntry(cache, entry.entry, last);
-	for (; entry.entry < (signed int) last; ++entry.entry) {
-#ifdef DEBUG
-		fprintf(stderr,"Clearing entry %d.\n", entry.entry);
-#endif
-		dir_read(&entry, &error);
-		if(error)
-			return error;
-		if(!entry.dir.name[0] || entry.dir.name[0] == DELMARK)
-			break;
-		entry.dir.name[0] = DELMARK;
-		if (entry.dir.attr == 0xf)
-			entry.dir.attr = '\0';
-		low_level_dir_write(&entry);
-	}
-	return 0;
-}
-
-int write_vfat(Stream_t *Dir, dos_name_t *shortname, char *longname,
-	       unsigned int start,
-	       direntry_t *mainEntry)
+unsigned int write_vfat(Stream_t *Dir, dos_name_t *shortname, char *longname,
+			unsigned int start,
+			direntry_t *mainEntry)
 {
 	struct vfat_subentry *vse;
 	uint8_t vse_id, num_vses;
@@ -347,7 +316,7 @@ int write_vfat(Stream_t *Dir, dos_name_t *shortname, char *longname,
 		       vse->sum,shortname->base,shortname->ext);
 #endif
 
-		wlen = native_to_wchar(longname, wlongname, MAX_VNAMELEN+1,
+		wlen = native_to_wchar(longname, wlongname, MAX_VNAMELEN,
 				       0, 0);
 		num_vses = (uint8_t)((wlen + VSE_NAMELEN - 1)/VSE_NAMELEN);
 		for (vse_id = num_vses; vse_id; --vse_id) {
@@ -366,7 +335,7 @@ int write_vfat(Stream_t *Dir, dos_name_t *shortname, char *longname,
 			       start + num_vses - vse_id, start + num_vses);
 #endif
 
-			entry.entry = start + num_vses - vse_id;
+			setEntryToPos(&entry, start + num_vses - vse_id);
 			low_level_dir_write(&entry);
 		}
 	} else {
@@ -390,12 +359,12 @@ void dir_write(direntry_t *entry)
 	dirCacheEntry_t *dce;
 	dirCache_t *cache;
 
-	if(entry->entry == -3) {
+	if(isRootEntry(entry)) {
 		fprintf(stderr, "Attempt to write root directory pointer\n");
 		exit(1);
 	}
 
-	cache = allocDirCache(entry->Dir, entry->entry + 1);
+	cache = allocDirCache(entry->Dir, getNextEntryAsPos(entry));
 	if(!cache) {
 		fprintf(stderr, "Out of memory error in dir_write\n");
 		exit(1);
@@ -495,7 +464,7 @@ static dirCacheEntry_t *vfat_lookup_loop_common(doscp_t *cp,
 						int *io_error)
 {
 	wchar_t newfile[13];
-	unsigned int initpos = direntry->entry + 1;
+	unsigned int initpos = getNextEntryAsPos(direntry);
 	struct vfat_state vfat;
 	wchar_t *longname;
 	int error;
@@ -511,9 +480,10 @@ static dirCacheEntry_t *vfat_lookup_loop_common(doscp_t *cp,
 			    *io_error = error;
 			    return NULL;
 			}
-			addFreeEndEntry(cache, initpos, direntry->entry,
+			addFreeEndEntry(cache, initpos,
+					getEntryAsPos(direntry),
 					endmarkSeen);
-			return addEndEntry(cache, direntry->entry);
+			return addEndEntry(cache, getEntryAsPos(direntry));
 		}
 
 		if (endmarkSeen || direntry->dir.name[0] == ENDMARK){
@@ -522,7 +492,7 @@ static dirCacheEntry_t *vfat_lookup_loop_common(doscp_t *cp,
 				endmarkSeen = 1;
 				continue;
 			}
-			return addEndEntry(cache, direntry->entry);
+			return addEndEntry(cache, getEntryAsPos(direntry));
 		}
 		if(direntry->dir.name[0] != DELMARK &&
 		   direntry->dir.attr == 0x0f)
@@ -539,7 +509,7 @@ static dirCacheEntry_t *vfat_lookup_loop_common(doscp_t *cp,
 	/* deleted file */
 	if (direntry->dir.name[0] == DELMARK) {
 		return addFreeEntry(cache, initpos,
-				    direntry->entry + 1);
+				    getNextEntryAsPos(direntry));
 	}
 
 	check_vfat(&vfat, &direntry->dir);
@@ -548,7 +518,7 @@ static dirCacheEntry_t *vfat_lookup_loop_common(doscp_t *cp,
 
 	/* mark space between last entry and this one as free */
 	addFreeEntry(cache, initpos,
-		     direntry->entry - vfat.subentries);
+		     getEntryAsPos(direntry) - vfat.subentries);
 
 	if (direntry->dir.attr & 0x8){
 		/* Read entry as a label */
@@ -573,8 +543,8 @@ static dirCacheEntry_t *vfat_lookup_loop_common(doscp_t *cp,
 	else
 		longname = 0;
 
-	return addUsedEntry(cache, direntry->entry - vfat.subentries,
-			    direntry->entry + 1, longname,
+	return addUsedEntry(cache, getEntryAsPos(direntry) - vfat.subentries,
+			    getNextEntryAsPos(direntry), longname,
 			    newfile, &direntry->dir);
 }
 
@@ -589,7 +559,7 @@ static inline dirCacheEntry_t *vfat_lookup_loop_for_read(doscp_t *cp,
 	*io_error = 0;
 	dce = cache->entries[initpos];
 	if(dce) {
-		direntry->entry = dce->endSlot - 1;
+		setEntryToPos(direntry, dce->endSlot - 1);
 		return dce;
 	} else {
 		return vfat_lookup_loop_common(cp,
@@ -625,10 +595,17 @@ static result_t checkNameForMatch(struct direntry_t *direntry,
 		case DCET_USED:
 			break;
 #ifdef DEBUG
+# if defined HAVE_PRAGMA_DIAGNOSTIC && defined __clang__
+#  pragma clang diagnostic push
+#  pragma clang diagnostic ignored "-Wcovered-switch-default"
+# endif
 		default:
 			fprintf(stderr, "Unexpected entry type %d\n",
 				dce->type);
 			return RES_ERROR;
+# if defined HAVE_PRAGMA_DIAGNOSTIC && defined __clang__
+#  pragma clang diagnostic pop
+# endif
 #endif
 	}
 
@@ -708,10 +685,10 @@ int vfat_lookup(direntry_t *direntry, const char *filename,
 	else
 		length = 0;
 
-	if (direntry->entry == -2)
+	if (isNotFound(direntry))
 		return -1;
 
-	cache = allocDirCache(direntry->Dir, direntry->entry+1);
+	cache = allocDirCache(direntry->Dir, getNextEntryAsPos(direntry));
 	if(!cache) {
 		fprintf(stderr, "Out of memory error in vfat_lookup [0]\n");
 		exit(1);
@@ -745,7 +722,7 @@ int vfat_lookup(direntry_t *direntry, const char *filename,
 		direntry->endSlot = dce->endSlot-1;
 		return 0; /* file found */
 	} else {
-		direntry->entry = -2;
+		direntry->entry = NOT_FOUND_ENTRY;
 		return -1; /* no file found */
 	}
 }
@@ -762,7 +739,7 @@ static inline dirCacheEntry_t *vfat_lookup_loop_for_insert(doscp_t *cp,
 	if(dce && dce->type != DCET_END) {
 		return dce;
 	} else {
-		direntry->entry = initpos - 1;
+		setEntryForIteration(direntry, initpos);
 		dce = vfat_lookup_loop_common(cp,
 					      direntry, cache, 1, &io_error);
 		if(!dce) {
@@ -796,7 +773,7 @@ static void accountFreeSlots(struct scan_state *ssp, dirCacheEntry_t *dce)
 static void clear_scan(wchar_t *longname, int use_longname,
 		       struct scan_state *s)
 {
-	s->shortmatch = s->longmatch = s->slot = -1;
+	s->shortmatch = s->longmatch = -1;
 	s->free_end = s->got_slots = s->free_start = 0;
 
 	if (use_longname & 1)
@@ -813,26 +790,34 @@ static void clear_scan(wchar_t *longname, int use_longname,
 int lookupForInsert(Stream_t *Dir,
 		    struct direntry_t *direntry,
 		    dos_name_t *dosname,
-		    char *longname,
+		    const char *longname,
 		    struct scan_state *ssp,
 		    int ignore_entry,
 		    int source_entry,
 		    int pessimisticShortRename,
-		    int use_longname)
+		    int use_longname,
+		    int ignore_match)
 {
 	direntry_t entry;
-	int ignore_match;
+	int ignore_short_match;
 	dirCacheEntry_t *dce;
 	dirCache_t *cache;
 	unsigned int pos; /* position _before_ the next answered entry */
 	wchar_t shortName[13];
-	wchar_t wlongname[MAX_VNAMELEN+1];
+	wchar_t wlongname[MAX_VNAMELEN+2];
 	doscp_t *cp = GET_DOSCONVERT(Dir);
 
-	native_to_wchar(longname, wlongname, MAX_VNAMELEN+1, 0, 0);
+	/* Deliberately convert one character too much in order
+	 * to catch file names that are too long */
+	if(native_to_wchar(longname, wlongname, MAX_VNAMELEN+1, 0, 0) >
+	   MAX_VNAMELEN) {
+	    fprintf(stderr, "Name %s too long\n", longname);
+	    return 0; /* Single file error */
+	}
+	    
 	clear_scan(wlongname, use_longname, ssp);
 
-	ignore_match = (ignore_entry == -2 );
+	ignore_short_match = (ignore_entry == -2 ) || ignore_match;
 
 	initializeDirentry(&entry, Dir);
 	ssp->match_free = 0;
@@ -846,14 +831,14 @@ int lookupForInsert(Stream_t *Dir,
 		exit(1);
 	}
 
-	if(!ignore_match)
+	if(!ignore_short_match)
 		unix_name(cp, dosname->base, dosname->ext, 0, shortName);
 
 	pos = cache->nrHashed;
 	if(source_entry >= 0 ||
 	   (pos && isHashed(cache, wlongname))) {
 		pos = 0;
-	} else if(pos && !ignore_match && isHashed(cache, shortName)) {
+	} else if(pos && !ignore_short_match && isHashed(cache, shortName)) {
 		if(pessimisticShortRename) {
 			ssp->shortmatch = -2;
 			return 1;
@@ -880,12 +865,16 @@ int lookupForInsert(Stream_t *Dir,
 				    ((signed int)dce->endSlot-1==ignore_entry))
 					break;
 
+				if(ignore_match)
+				    break;
+
 				/* check long name */
 				if((dce->longName &&
 				    !wcscasecmp(dce->longName, wlongname)) ||
 				   (dce->shortName &&
 				    !wcscasecmp(dce->shortName, wlongname))) {
-					ssp->longmatch = dce->endSlot - 1;
+					ssp->longmatch =
+						(int) (dce->endSlot - 1);
 					/* long match is a reason for
 					 * immediate stop */
 					direntry->beginSlot = dce->beginSlot;
@@ -895,9 +884,10 @@ int lookupForInsert(Stream_t *Dir,
 
 				/* Long name or not, always check for
 				 * short name match */
-				if (!ignore_match &&
+				if (!ignore_short_match &&
 				    !wcscasecmp(shortName, dce->shortName))
-					ssp->shortmatch = dce->endSlot - 1;
+					ssp->shortmatch =
+						(int) (dce->endSlot - 1);
 				break;
 			case DCET_END:
 				break;
@@ -917,9 +907,6 @@ int lookupForInsert(Stream_t *Dir,
 	fprintf(stderr, "No directory slots\n");
 	return -1;
 }
-#ifdef HAVE_PRAGMA_DIAGNOSTIC
-# pragma GCC diagnostic pop
-#endif
 
 
 /* End vfat.c */
